/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * dcc_offer.h -- Part of the ezbounce IRC proxy
 * (C) 2007 Murat Deligonul
 */

#ifndef __DCC_OFFER_H
#define __DCC_OFFER_H

#include <string>
#include "util/tracked_object.h"
#include "util/expirable_object.h"
#include "util/managed_table.h"

class conn;

class dcc_offer : public util::tracked_object<dcc_offer>, public util::expirable_object<dcc_offer>
{
public:
	enum dcc_offer_flag_t {
		DEAD     = 0,
		INCOMING = 1,
		OUTGOING = 2,
		RESUMABLE = 4,
	};
	
	enum dcc_offer_info_t {
		FROM = 0,
		TO  ,
		TYPE,
		ADDRESS,
		FILENAME,
		NUM_STRINGS
	};

private:
	util::managed_table<std::string, NUM_STRINGS> 		data;
	conn * 							_owner;
	int 							_flags;
	unsigned int 						_id;
	unsigned short 						_port;
	unsigned int 						_size;
	unsigned int 						_offset;
	time_t 							_start;
	
public:
	dcc_offer(conn * owner, const char * type, const char * from, const char * to,
			const char * address, unsigned short, const char * file,
			unsigned int size, unsigned int offset, int f);
	~dcc_offer();
	
	int flags() const { return _flags; }
	unsigned int id() const { return _id; }
	unsigned short port() const { return _port; }
	unsigned int   size() const { return _size; }
	unsigned int   offset() const { return _offset; }
	const char * type() const { return data.get(TYPE).c_str(); }
	const char * to() const { return data.get(TO).c_str(); }
	const char * from() const { return data.get(FROM).c_str(); }
	const char * address() const { return data.get(ADDRESS).c_str(); }
	const char * filename() const { return data.get(FILENAME).c_str(); }
	conn * owner() { return _owner; }
	time_t age() const { return ircproxy::instance()->time() - _start; }
	time_t start_time() const { return _start; }
	
	void set(dcc_offer_info_t i, const char * str) { data.set(i, str); }
	void set_flags(int f)  { _flags = f; }
	void set_id(unsigned int i) { _id = i; }
	void set_port(unsigned short p) { _port = p; }
	void set_owner(conn * o);
	int  info(char *, int);
	int  info(conn *);

	/**
	 * expirable_object<T> interface requirements 
	 */	
	int status_check(time_t, const void ** = NULL) const;
	int die(int, const void *);

private:
	dcc_offer(const dcc_offer&);
	dcc_offer& operator = (const dcc_offer&);
};

#endif
